@@ -19,12 +19,25 @@ from mlflow.tracking import artifact_utils, _get_store
 from mlflow.tracking.context import registry as context_registry
 from mlflow.store.tracking import SEARCH_MAX_RESULTS_DEFAULT
 from mlflow.utils import env
+from mlflow.utils.autologging_utils import _is_testing, autologging_integration
 from mlflow.utils.databricks_utils import is_in_databricks_notebook, get_notebook_id
 from mlflow.utils.import_hooks import register_post_import_hook
 from mlflow.utils.mlflow_tags import MLFLOW_PARENT_RUN_ID, MLFLOW_RUN_NAME
 from mlflow.utils.validation import _validate_run_id
+from mlflow.utils.annotations import experimental
 
-from mlflow import tensorflow, keras, gluon, xgboost, lightgbm, spark, sklearn, fastai, pytorch
+from mlflow import (
+    tensorflow,
+    keras,
+    gluon,
+    xgboost,
+    lightgbm,
+    statsmodels,
+    spark,
+    sklearn,
+    fastai,
+    pytorch,
+)
 
 _EXPERIMENT_ID_ENV_VAR = "MLFLOW_EXPERIMENT_ID"
 _EXPERIMENT_NAME_ENV_VAR = "MLFLOW_EXPERIMENT_NAME"
@@ -99,7 +112,7 @@ class ActiveRun(Run):  # pylint: disable=W0223
         return exc_type is None
 
 
-def start_run(run_id=None, experiment_id=None, run_name=None, nested=False):
+def start_run(run_id=None, experiment_id=None, run_name=None, nested=False, tags=None):
     """
     Start a new MLflow run, setting it as the active run under which metrics and parameters
     will be logged. The return value can be used as a context manager within a ``with`` block;
@@ -126,7 +139,8 @@ def start_run(run_id=None, experiment_id=None, run_name=None, nested=False):
                           or the default experiment as defined by the tracking server.
     :param run_name: Name of new run (stored as a ``mlflow.runName`` tag).
                      Used only when ``run_id`` is unspecified.
-    :param nested: Controls whether run is nested in parent run. ``True`` creates a nest run.
+    :param nested: Controls whether run is nested in parent run. ``True`` creates a nested run.
+    :param tags: An optional dictionary of string keys and values to set as tags on the new run.
     :return: :py:class:`mlflow.ActiveRun` object that acts as a context manager wrapping
              the run's state.
 
@@ -212,7 +226,7 @@ def start_run(run_id=None, experiment_id=None, run_name=None, nested=False):
 
         exp_id_for_run = experiment_id if experiment_id is not None else _get_experiment_id()
 
-        user_specified_tags = {}
+        user_specified_tags = tags or {}
         if parent_run_id is not None:
             user_specified_tags[MLFLOW_PARENT_RUN_ID] = parent_run_id
         if run_name is not None:
@@ -557,6 +571,176 @@ def log_artifacts(local_dir, artifact_path=None):
     MlflowClient().log_artifacts(run_id, local_dir, artifact_path)
 
 
+def log_text(text, artifact_file):
+    """
+    Log text as an artifact.
+
+    :param text: String containing text to log.
+    :param artifact_file: The run-relative artifact file path in posixpath format to which
+                          the text is saved (e.g. "dir/file.txt").
+
+    .. code-block:: python
+        :caption: Example
+
+        import mlflow
+
+        with mlflow.start_run():
+            # Log text to a file under the run's root artifact directory
+            mlflow.log_text("text1", "file1.txt")
+
+            # Log text in a subdirectory of the run's root artifact directory
+            mlflow.log_text("text2", "dir/file2.txt")
+
+            # Log HTML text
+            mlflow.log_text("<h1>header</h1>", "index.html")
+    """
+    run_id = _get_or_start_run().info.run_id
+    MlflowClient().log_text(run_id, text, artifact_file)
+
+
+@experimental
+def log_dict(dictionary, artifact_file):
+    """
+    Log a dictionary as an artifact. The serialization format (JSON or YAML) is automatically
+    inferred from the extension of `artifact_file`. If the file extension doesn't exist or match
+    any of [".json", ".yml", ".yaml"], JSON format is used.
+
+    :param dictionary: Dictionary to log.
+    :param artifact_file: The run-relative artifact file path in posixpath format to which
+                            the dictionary is saved (e.g. "dir/data.json").
+
+    .. code-block:: python
+        :caption: Example
+
+        import mlflow
+
+        dictionary = {"k": "v"}
+
+        with mlflow.start_run():
+            # Log a dictionary as a JSON file under the run's root artifact directory
+            mlflow.log_dict(dictionary, "data.json")
+
+            # Log a dictionary as a YAML file in a subdirectory of the run's root artifact directory
+            mlflow.log_dict(dictionary, "dir/data.yml")
+
+            # If the file extension doesn't exist or match any of [".json", ".yaml", ".yml"],
+            # JSON format is used.
+            mlflow.log_dict(dictionary, "data")
+            mlflow.log_dict(dictionary, "data.txt")
+    """
+    run_id = _get_or_start_run().info.run_id
+    MlflowClient().log_dict(run_id, dictionary, artifact_file)
+
+
+@experimental
+def log_figure(figure, artifact_file):
+    """
+    Log a figure as an artifact. The following figure objects are supported:
+
+    - `matplotlib.figure.Figure`_
+    - `plotly.graph_objects.Figure`_
+
+    .. _matplotlib.figure.Figure:
+        https://matplotlib.org/api/_as_gen/matplotlib.figure.Figure.html
+
+    .. _plotly.graph_objects.Figure:
+        https://plotly.com/python-api-reference/generated/plotly.graph_objects.Figure.html
+
+    :param figure: Figure to log.
+    :param artifact_file: The run-relative artifact file path in posixpath format to which
+                          the figure is saved (e.g. "dir/file.png").
+
+    .. code-block:: python
+        :caption: Matplotlib Example
+
+        import mlflow
+        import matplotlib.pyplot as plt
+
+        fig, ax = plt.subplots()
+        ax.plot([0, 1], [2, 3])
+
+        with mlflow.start_run():
+            mlflow.log_figure(fig, "figure.png")
+
+    .. code-block:: python
+        :caption: Plotly Example
+
+        import mlflow
+        from plotly import graph_objects as go
+
+        fig = go.Figure(go.Scatter(x=[0, 1], y=[2, 3]))
+
+        with mlflow.start_run():
+            mlflow.log_figure(fig, "figure.html")
+    """
+    run_id = _get_or_start_run().info.run_id
+    MlflowClient().log_figure(run_id, figure, artifact_file)
+
+
+@experimental
+def log_image(image, artifact_file):
+    """
+    Log an image as an artifact. The following image objects are supported:
+
+    - `numpy.ndarray`_
+    - `PIL.Image.Image`_
+
+    .. _numpy.ndarray:
+        https://numpy.org/doc/stable/reference/generated/numpy.ndarray.html
+
+    .. _PIL.Image.Image:
+        https://pillow.readthedocs.io/en/stable/reference/Image.html#PIL.Image.Image
+
+    Numpy array support
+        - data type (( ) represents a valid value range):
+
+            - bool
+            - integer (0 ~ 255)
+            - unsigned integer (0 ~ 255)
+            - float (0.0 ~ 1.0)
+
+            .. warning::
+
+                - Out-of-range integer values will be **clipped** to [0, 255].
+                - Out-of-range float values will be **clipped** to [0, 1].
+
+        - shape (H: height, W: width):
+
+            - H x W (Grayscale)
+            - H x W x 1 (Grayscale)
+            - H x W x 3 (an RGB channel order is assumed)
+            - H x W x 4 (an RGBA channel order is assumed)
+
+    :param image: Image to log.
+    :param artifact_file: The run-relative artifact file path in posixpath format to which
+                          the image is saved (e.g. "dir/image.png").
+
+    .. code-block:: python
+        :caption: Numpy Example
+
+        import mlflow
+        import numpy as np
+
+        image = np.random.randint(0, 256, size=(100, 100, 3), dtype=np.uint8)
+
+        with mlflow.start_run():
+            mlflow.log_image(image, "image.png")
+
+    .. code-block:: python
+        :caption: Pillow Example
+
+        import mlflow
+        from PIL import Image
+
+        image = Image.new("RGB", (100, 100))
+
+        with mlflow.start_run():
+            mlflow.log_image(image, "image.png")
+    """
+    run_id = _get_or_start_run().info.run_id
+    MlflowClient().log_image(run_id, image, artifact_file)
+
+
 def _record_logged_model(mlflow_model):
     run_id = _get_or_start_run().info.run_id
     MlflowClient()._record_logged_model(run_id, mlflow_model)
@@ -1027,9 +1211,16 @@ def _get_experiment_id():
     ) or deprecated_default_exp_id
 
 
-def autolog(log_input_examples=False, log_model_signatures=True):  # pylint: disable=unused-argument
+@autologging_integration("mlflow")
+def autolog(
+    log_input_examples=False,
+    log_model_signatures=True,
+    log_models=True,
+    disable=False,
+    exclusive=False,
+):  # pylint: disable=unused-argument
     """
-    Enable autologging for all supported integrations.
+    Enables (or disables) and configures autologging for all supported integrations.
 
     The parameters are passed to any autologging integrations that support them.
 
@@ -1039,11 +1230,70 @@ def autolog(log_input_examples=False, log_model_signatures=True):  # pylint: dis
     :param log_input_examples: If ``True``, input examples from training datasets are collected and
                                logged along with model artifacts during training. If ``False``,
                                input examples are not logged.
+                               Note: Input examples are MLflow model attributes
+                               and are only collected if ``log_models`` is also ``True``.
     :param log_model_signatures: If ``True``,
                                  :py:class:`ModelSignatures <mlflow.models.ModelSignature>`
                                  describing model inputs and outputs are collected and logged along
                                  with model artifacts during training. If ``False``, signatures are
-                                 not logged.
+                                 not logged. Note: Model signatures are MLflow model attributes
+                                 and are only collected if ``log_models`` is also ``True``.
+    :param log_models: If ``True``, trained models are logged as MLflow model artifacts.
+                       If ``False``, trained models are not logged.
+                       Input examples and model signatures, which are attributes of MLflow models,
+                       are also omitted when ``log_models`` is ``False``.
+    :param disable: If ``True``, disables all supported autologging integrations. If ``False``,
+                    enables all supported autologging integrations.
+    :param exclusive: If ``True``, autologged content is not logged to user-created fluent runs.
+                      If ``False``, autologged content is logged to the active fluent run,
+                      which may be user-created.
+
+    .. code-block:: python
+        :caption: Example
+
+        import numpy as np
+        import mlflow.sklearn
+        from mlflow.tracking import MlflowClient
+        from sklearn.linear_model import LinearRegression
+
+        def print_auto_logged_info(r):
+            tags = {k: v for k, v in r.data.tags.items() if not k.startswith("mlflow.")}
+            artifacts = [f.path for f in MlflowClient().list_artifacts(r.info.run_id, "model")]
+            print("run_id: {}".format(r.info.run_id))
+            print("artifacts: {}".format(artifacts))
+            print("params: {}".format(r.data.params))
+            print("metrics: {}".format(r.data.metrics))
+            print("tags: {}".format(tags))
+
+        # prepare training data
+        X = np.array([[1, 1], [1, 2], [2, 2], [2, 3]])
+        y = np.dot(X, np.array([1, 2])) + 3
+
+        # Auto log all the parameters, metrics, and artifacts
+        mlflow.autolog()
+        model = LinearRegression()
+        with mlflow.start_run() as run:
+            model.fit(X, y)
+
+        # fetch the auto logged parameters and metrics for ended run
+        print_auto_logged_info(mlflow.get_run(run_id=run.info.run_id))
+
+    .. code-block:: text
+        :caption: Output
+
+        run_id: fd10a17d028c47399a55ab8741721ef7
+        artifacts: ['model/MLmodel', 'model/conda.yaml', 'model/model.pkl']
+        params: {'copy_X': 'True',
+                 'normalize': 'False',
+                 'fit_intercept': 'True',
+                 'n_jobs': 'None'}
+        metrics: {'training_score': 1.0,
+                  'training_rmse': 4.440892098500626e-16,
+                  'training_r2_score': 1.0,
+                  'training_mae': 2.220446049250313e-16,
+                  'training_mse': 1.9721522630525295e-31}
+        tags: {'estimator_class': 'sklearn.linear_model._base.LinearRegression',
+               'estimator_name': 'LinearRegression'}
     """
     locals_copy = locals().items()
 
@@ -1055,6 +1305,7 @@ def autolog(log_input_examples=False, log_model_signatures=True):  # pylint: dis
         "mxnet.gluon": gluon.autolog,
         "xgboost": xgboost.autolog,
         "lightgbm": lightgbm.autolog,
+        "statsmodels": statsmodels.autolog,
         "sklearn": sklearn.autolog,
         "fastai": fastai.autolog,
         "pyspark": spark.autolog,
@@ -1063,21 +1314,30 @@ def autolog(log_input_examples=False, log_model_signatures=True):  # pylint: dis
         "pytorch_lightning": pytorch.autolog,
     }
 
-    def setup_autologging(module):
-        autolog_fn = LIBRARY_TO_AUTOLOG_FN[module.__name__]
+    def get_autologging_params(autolog_fn):
         try:
             needed_params = list(inspect.signature(autolog_fn).parameters.keys())
-            filtered = {k: v for k, v in locals_copy if k in needed_params}
-        except ValueError:
-            filtered = {}
+            return {k: v for k, v in locals_copy if k in needed_params}
+        except Exception:  # pylint: disable=broad-except
+            return {}
 
+    def setup_autologging(module):
         try:
-            autolog_fn(**filtered)
+            autolog_fn = LIBRARY_TO_AUTOLOG_FN[module.__name__]
+            autologging_params = get_autologging_params(autolog_fn)
+            autolog_fn(**autologging_params)
             _logger.info("Autologging successfully enabled for %s.", module.__name__)
         except Exception as e:  # pylint: disable=broad-except
-            _logger.warning(
-                "Exception raised while enabling autologging for %s: %s", module.__name__, str(e)
-            )
+            if _is_testing():
+                # Raise unexpected exceptions in test mode in order to detect
+                # errors within dependent autologging integrations
+                raise
+            else:
+                _logger.warning(
+                    "Exception raised while enabling autologging for %s: %s",
+                    module.__name__,
+                    str(e),
+                )
 
     # for each autolog library (except pyspark), register a post-import hook.
     # this way, we do not send any errors to the user until we know they are using the library.
@@ -1089,11 +1349,17 @@ def autolog(log_input_examples=False, log_model_signatures=True):  # pylint: dis
     # this is because on Databricks a SparkSession already exists and the user can directly
     #   interact with it, and this activity should be logged.
     try:
-        spark.autolog()
+        autologging_params = get_autologging_params(spark.autolog)
+        spark.autolog(**autologging_params)
     except ImportError as ie:
         # if pyspark isn't installed, a user could potentially install it in the middle
         #   of their session so we want to enable autologging once they do
         if "pyspark" in str(ie):
             register_post_import_hook(setup_autologging, "pyspark", overwrite=True)
     except Exception as e:  # pylint: disable=broad-except
-        _logger.warning("Exception raised while enabling autologging for spark: %s", str(e))
+        if _is_testing():
+            # Raise unexpected exceptions in test mode in order to detect
+            # errors within dependent autologging integrations
+            raise
+        else:
+            _logger.warning("Exception raised while enabling autologging for spark: %s", str(e))
